import os
import shutil
import sys
import venv
from typing import List

__func_alias__ = {"bin_": "bin"}


def bin_(hub, bname: str) -> List[str]:
    """

    :param hub:
    :param bname:
    :return:
    """
    opts = hub.tiamat.BUILDS[bname]
    if os.path.isfile(opts.pybin):
        return [opts.pybin]
    elif getattr(sys, "frozen", False):
        # If sys.frozen is set, then we are running in a tiamat binary
        hub.log.debug("Running inside a binary, searching for system python")
        return [shutil.which("python3")]
    else:
        hub.log.debug("Using system executable to build virtualenv")
        return [sys.executable]


def create(hub, bname: str):
    """
    Change up the opts for this build so that they use the appropriate system python
    directories

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    pybin = hub.tiamat.virtualenv.builtin.bin(bname)

    if hasattr(sys, "frozen"):
        # some venv functions need to be overriden if we are running from a tiamat
        # binary
        sys._base_executable = pybin[0]

    builder = venv.EnvBuilder(
        system_site_packages=opts.sys_site,
        clear=True,
        symlinks=False,
        upgrade=True,
        with_pip=True,
        prompt=None,
    )
    builder.create(opts.venv_dir)

    # Reset sys base executable
    if hasattr(sys, "frozen"):
        sys._base_executable = sys.executable



import os
import shutil
import tempfile
from typing import Dict
from typing import Iterable
from typing import List
from typing import Set


def build(hub, bname: str):
    """
    Build a new package using fpm.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    root = os.path.join(tempfile.mkdtemp(), opts.name)
    os.makedirs(root)
    # Move files into a tempdir, return config files
    # Set the specific flags, like checksum type, based on the pkg_tgt
    # Run fpm
    config = _prep_tmpdir(
        hub,
        opts.name,
        root,
        opts.pkg,
        opts.pkg_tgt,
        opts.dir,
        opts.onedir,
    )
    cmd = _get_cmd(
        opts.name,
        root,
        opts.dependencies,
        opts.release,
        opts.pkg,
        opts.pkg_tgt,
        opts.version,
        config,
    )
    hub.tiamat.cmd.run(cmd, fail_on_error=True)
    # shutil.rmtree(root)


def _get_fpm_tgt(pkg_tgt: str):
    """
    Turn the pkg_tgt into a target for fpm.

    :param pkg_tgt: The os/distribution target to build the package against.
    """
    rpm = (
        "rpm",
        "fedora",
        "redhat",
        "rhel",
        "opensuse",
        "suse",
        "cent",
        "centos",
        "sles",
        "sle",
    )
    deb = ("ubuntu", "deb")
    pacman = ("arch", "manjaro", "pacman")
    if pkg_tgt.lower().startswith(rpm):
        return "rpm"
    elif pkg_tgt.startswith(deb):
        return "deb"
    elif pkg_tgt.lower().startswith(pacman):
        return "pacman"
    else:
        raise ValueError(f"invalid pkg target: {pkg_tgt}")


def _get_cmd(
    name: str,
    root: str,
    dependencies: Iterable[str],
    release: str,
    pkg: Dict[str, str],
    pkg_tgt: str,
    version: str,
    config: Iterable[str],
) -> List[str]:
    """
    Return the command line args list to shell out to fpm with.

    :param name: The name of the project to build.
    :param root: The path to the directory to build from.
    :param dependencies: comma separated list of dependencies needed for the build.
    :param release: This is the fpm --iteration option i.e. "1.el7".
    :param pkg: Options for building packages.
    :param pkg_tgt: The os/distribution target to build the package against.
    :param version: Target package version.
    :param config: Load extra options from a configuration file.
    """
    version = str(pkg.get("version", version))
    fpm_tgt = _get_fpm_tgt(pkg_tgt)
    fpm = shutil.which("fpm")
    if not fpm:
        raise OSError("fpm command is not available")
    cmd = [fpm, "-s", "dir", "-n", name, "-t", fpm_tgt]

    for fn in sorted(list(config)):
        cmd.append("--config-files")
        cmd.append(fn)
    if fpm_tgt == "rpm":
        cmd.append("--rpm-digest")
        cmd.append("sha512")
    if dependencies:
        for dependency in dependencies:
            cmd.append("-d")
            cmd.append(f"{dependency}")
    if release:
        cmd.append("--iteration")
        cmd.append(f"{release}")
    if pkg.get("after-install"):
        cmd.append("--after-install")
        cmd.append(pkg.get("after-install"))
    if pkg.get("before-install"):
        cmd.append("--before-install")
        cmd.append(pkg.get("before-install"))
    if pkg.get("after-remove"):
        cmd.append("--after-remove")
        cmd.append(pkg.get("after-remove"))
    if pkg.get("before-remove"):
        cmd.append("--before-remove")
        cmd.append(pkg.get("before-remove"))
    if pkg.get("after-upgrade"):
        cmd.append("--after-upgrade")
        cmd.append(pkg.get("after-upgrade"))
    if pkg.get("before-upgrade"):
        cmd.append("--before-upgrade")
        cmd.append(pkg.get("before-upgrade"))
    cmd.append("--version")
    cmd.append(version)
    cmd.append("-C")
    cmd.append(root)
    return cmd


def _prep_tmpdir(
    hub,
    name: str,
    root: str,
    pkg: Dict[str, str],
    pkg_tgt: str,
    dir_: str,
    onedir: bool,
) -> Set[str]:
    """
    Make the tempdir and copy the configured files into it.

    :param hub:
    :param name:
    :param root:
    :param pkg:
    :param pkg_tgt:
    :param dir_:
    :param onedir:
    :return:
    """
    script_template = """#!/bin/sh
    {bin_path} $@"""
    dist = os.path.join(dir_, "dist", name)
    configs = set()
    bin_tgt = os.path.join(root, "usr", "bin")
    if onedir:
        dist = os.path.join(dir_, "dist", "run")
        tree_tgt = os.path.join(root, "opt", "run", "bins")
        pbin = os.path.join(f"{os.sep}opt", "run", "bins", "run")
        os.makedirs(os.path.dirname(tree_tgt))
        shutil.copytree(dist, tree_tgt)
        os.makedirs(bin_tgt)
        script_output = script_template.format(bin_path=pbin)
        with open(os.path.join(bin_tgt, name), "w") as script_file:
            script_file.write(script_output)
        os.chmod(os.path.join(bin_tgt, name), 0o755)
    else:
        os.makedirs(bin_tgt)
        shutil.copy(dist, bin_tgt)
    for tpath, spath in pkg.get("config", {}).items():
        tpath = tpath.strip(os.sep)
        src = os.path.join(dir_, spath)
        tgt = os.path.join(root, tpath)
        tgt_dir = os.path.dirname(tgt)
        if not os.path.isdir(tgt_dir):
            os.makedirs(tgt_dir)
        shutil.copy(src, tgt)
        configs.add(tpath)
    for spath in pkg.get("scripts", {}):
        src = os.path.join(dir_, spath)
        if not os.path.isdir(bin_tgt):
            os.makedirs(bin_tgt)
        shutil.copy(src, bin_tgt)
    if pkg_tgt in hub.tiamat.SYSTEMD:
        tgt = os.path.join(root, hub.tiamat.SYSTEMD_DIR)
        for spath in pkg.get("systemd", {}):
            if not os.path.isdir(tgt):
                os.makedirs(tgt)
            src = os.path.join(dir_, spath)
            shutil.copy(src, tgt)
    if pkg_tgt in hub.tiamat.SYSV:
        tgt = os.path.join(root, hub.tiamat.SYSV_DIR)
        for spath in pkg.get("sysv", {}):
            if not os.path.isdir(tgt):
                os.makedirs(tgt)
            src = os.path.join(dir_, spath)
            shutil.copy(src, tgt)
    return configs



import ast
import fnmatch
import os
import re
import shutil
import tarfile
from typing import Dict
from typing import List
from typing import Set

# "venv" is a keyword for bash, call the directory "virtualenv" but allow it to be
# accessed with the alias
__sub_alias__ = ["venv"]
__func_alias__ = {"bin_": "bin"}

OMIT = (
    "__pycache__",
    "PyInstaller",
)


def bin_(hub, bname: str) -> List[str]:
    func = getattr(hub.tiamat.virtualenv, hub.OPT.tiamat.venv_plugin).bin
    return func(bname)


def freeze(hub, bname: str) -> Set[str]:
    """
    List the requirements inside the given virtual environment sans those excluded by
    the build

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]

    # Freeze the modules installed into the environment
    ret = hub.tiamat.cmd.run(
        [opts.pybin, "-m", "pip", "freeze", "--all", "--exclude-editable"],
        cwd=opts.srcdir,
        timeout=opts.timeout,
        fail_on_error=True,
    )

    standardize_name = (
        lambda item: re.split("[= ]", item, maxsplit=1)[0].lower().replace("_", "-")
    )

    # Compare apples to apples, remove everything after "==" or " " from the excludes
    # and the freeze
    excluded = {standardize_name(ex) for ex in opts.exclude}

    bundled_modules = set()
    for module in ret.stdout.splitlines():
        if standardize_name(module) not in excluded:
            bundled_modules.add(module)

    return bundled_modules


def create(hub, bname: str):
    """
    Make a virtual environment based on the version of python used to call this script.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    func = getattr(hub.tiamat.virtualenv, opts.venv_plugin).create
    hub.log.debug(f"Creating virtual environment with {opts.venv_plugin}")
    func(bname)

    return hub.tiamat.virtualenv.init.setup_pip(bname)


def env(hub, bname) -> List[str]:
    opts = hub.tiamat.BUILDS[bname]
    ret = []
    if opts.locale_utf8:
        ret.extend(["env", "PYTHONUTF8=1", "LANG=POSIX"])
    return ret


def scan(hub, bname: str):
    """
    Scan the new venv for files and imports.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    omit = opts.omit or []
    for root, dirs, files in os.walk(opts.vroot):
        if _omit(root, omit):
            hub.log.trace(f"Ignoring root in {root}")
            continue
        for d in dirs:
            full = os.path.join(root, d)
            if _omit(full, omit):
                hub.log.trace(f"Ignoring dirs in {full}")
                continue
            opts.all_paths.add(full)
        for f in files:
            full = os.path.join(root, f)
            if _omit(full, omit):
                hub.log.trace(f"Ignoring full in {full}")
                continue
            opts.all_paths.add(full)


def mk_adds(hub, bname: str):
    """
    Make the imports and datas for PyInstaller.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    for path in opts.all_paths:
        if "site-packages" not in path:
            continue
        if os.path.isfile(path):
            if not path.endswith(".py"):
                continue
            if path.endswith("__init__.py"):
                # Skip it, we will get the dir
                continue
            imp = _to_import(path)
            if imp:
                opts.imports.add(imp)
        if os.path.isdir(path):
            data = _to_data(path)
            imp = _to_import(path)
            if imp:
                opts.imports.add(imp)
            if data:
                opts.datas.add(data)


def setup_pip(hub, bname: str):
    """
    Initialize pip in the new virtual environment

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    pip_cmd = [opts.pybin, "-m", "pip"]

    # Verify that an executable version of pip exists
    hub.tiamat.cmd.run(
        pip_cmd + ["--version"], timeout=opts.timeout, fail_on_error=True
    )

    if "latest" == opts.pip_version:
        pip_selection = "pip"
    else:
        pip_selection = f"pip=={opts.pip_version}"

    # Upgrade pip, setuptools, and wheel
    pre_reqs = [
        pip_selection,
        "pyinstaller-hooks-contrib",
        "setuptools>=50.3.0",
        "wheel",
        "pycparser",
    ]

    try:
        version_str = hub.tiamat.cmd.run(
            [opts["pybin"], "-c", "import sys;print(tuple(sys.version_info))"],
            shell=True,
            timeout=opts.timeout,
            fail_on_error=True,
        ).stdout

        version_info = ast.literal_eval(version_str)

        # distro is needed for Python3.8 and later. It replaces
        # platform.linux_distribution, which was included in Python until Python3.7
        if version_info >= (3, 8):
            pre_reqs.append("distro")
    except Exception as e:
        hub.log.error(f"Unable to determine target python version: {e}")

    # Install pre-requirements
    hub.tiamat.cmd.run(
        pip_cmd + ["install", "--upgrade"] + pre_reqs,
        timeout=opts.timeout,
        fail_on_error=True,
    )

    requirements = []
    if opts.get("req"):
        requirements.extend(["-r", opts.req])
    if opts.srcdir:
        files = _get_srcdir_files(opts.srcdir)
        requirements.extend(files)
    if os.path.isfile(os.path.join(opts.dir, "setup.py")):
        requirements.append(opts.dir)

    if ":" in opts.pyinstaller_version:
        pyinstall_version, pyinstall_path = opts.pyinstaller_version.split(":")
    else:
        pyinstall_version = opts.pyinstaller_version
        pyinstall_path = None

    if pyinstall_version == "dev":
        # Install development version of pyinstaller to run on python 3.8
        requirements.append(
            "https://github.com/pyinstaller/pyinstaller/tarball/develop"
        )
    elif pyinstall_version == "local":
        requirements.append("".join(pyinstall_path))
    else:
        requirements.append(f"PyInstaller=={pyinstall_version}")

    hub.tiamat.cmd.run(
        pip_cmd + ["install"] + requirements,
        cwd=opts.srcdir,
        timeout=opts.timeout,
        fail_on_error=True,
    )
    if opts.get("venv_uninstall"):
        hub.tiamat.cmd.run(
            pip_cmd + ["uninstall", "-y"] + list(opts.get("venv_uninstall")),
            cwd=opts.srcdir,
            timeout=opts.timeout,
            fail_on_error=True,
        )

    if opts.system_copy_in:
        _copy_in(hub, opts)

    ld_library_path = os.environ.get("LD_LIBRARY_PATH")
    venv_lib = os.path.join(opts.venv_dir, "lib")
    os.environ["LD_LIBRARY_PATH"] = f"{ld_library_path}:{venv_lib}".strip(":")
    # Add libpath to the environment for AIX
    os.environ[
        "LIBPATH"
    ] = f"{os.environ.get('LIBPATH')}:{os.path.join(opts.venv_dir, 'lib')}".strip(":")


def _omit(test: str, exlcude_patterns: List[str]) -> bool:
    """
    Check if a file should be omitted based on globs
    :param test: The string to compare to omit patterns
    :param exlcude_patterns: A glob to check against
    :return: True if the file should be omitted, else false
    """
    for bad in OMIT:
        if bad in test:
            return True

    return any(fnmatch.fnmatch(test, pattern) for pattern in exlcude_patterns)


def _to_import(path: str) -> str:
    """
    :param path:
    :return:
    """
    ret = path[path.index("site-packages") + 14 :].replace(os.sep, ".")
    if ret.endswith(".py"):
        ret = ret[:-3]
    return ret


def _to_data(path: str) -> str:
    """
    :param path:
    :return:
    """
    dest = path[path.index("site-packages") + 14 :]
    src = path
    if not dest.strip():
        return ""
    ret = f"{src}{os.pathsep}{dest}"
    return ret


def _copy_in(hub, opts: Dict[str, str]):
    """
    Copy in any extra directories from the python install.

    :param opts:
    """
    cmd = [opts["pybin"], "-c", "import sys;print(sys.path)"]
    tgt = ""
    dtgt = os.path.join(os.path.join(opts["venv_dir"], "lib"))
    for fn in os.listdir(dtgt):
        tmptgt = os.path.join(dtgt, fn)
        if os.path.isdir(tmptgt):
            tgt = os.path.join(tmptgt, "site-packages")

    data = hub.tiamat.cmd.run(
        cmd, shell=True, timeout=opts.timeout, fail_on_error=True
    ).stdout
    done = set()
    for path in ast.literal_eval(data):
        if not path:
            continue
        if not os.path.isdir(path):
            continue
        for fn in os.listdir(path):
            if fn in done:
                continue
            if fn in opts["system_copy_in"]:
                full = os.path.join(path, fn)
                if os.path.isdir(full):
                    shutil.copytree(full, os.path.join(tgt, fn))
                    done.add(fn)


def _get_srcdir_files(srcdir: str) -> List[str]:
    """
    Return the files that are python archives.

    :param srcdir:
    """
    files = []
    for fn in os.listdir(srcdir):
        if fn.endswith(".whl"):
            files.append(fn)
        if fn.endswith(".tar.gz"):
            # Might be a source archive
            with tarfile.open(fn) as tfp:
                for name in tfp.getnames():
                    if name.count(os.sep) > 1:
                        continue
                    if os.path.basename(name) == "PKG-INFO":
                        files.append(fn)
                        break
    return files



import os
import shlex
import subprocess
import sys
from typing import Any
from typing import Dict
from typing import List

from dict_tools import data

__virtualname__ = "cmd"


def run(
    hub,
    cmd: List[str] or str,
    shell: bool = False,
    env: Dict[str, str] = None,
    cwd: str = None,
    timeout: int = 300,
    fail_on_error: bool = False,
    **kwargs,
) -> Dict[str, Any]:
    """
    Run every command the same way
    :param hub:
    :param cmd:
    :param shell:
    :param env:
    :param cwd:
    :param timeout:
    :param fail_on_error:
    :return:
    """
    if env is None and getattr(sys, "frozen", False):
        env = os.environ.copy()
        # Remove the LOAD LIBRARY_PATH for running commands
        # https://pyinstaller.readthedocs.io/en/stable/runtime-information.html#ld-library-path-libpath-considerations
        env.pop("DYLD_LIBRARY_PATH", None)  # Darwin
        env.pop("LD_LIBRARY_PATH", None)  # Linux
        env.pop("LIBPATH", None)  # AIX

    if cwd is None:
        cwd = os.getcwd()

    if shell is True:
        if isinstance(cmd, str):
            cmd = " ".join(c for c in shlex.split(cmd))
        else:
            cmd = " ".join(shlex.quote(c) for c in cmd)
    elif isinstance(cmd, str):
        cmd = cmd.split()

    dry_run = hub.OPT.tiamat.dry_run
    if dry_run:
        hub.log.debug(f"Dry run command: {cmd}")
        return data.NamespaceDict(
            retcode=0, stdout="()", stderr=f"Dry Run: {cmd}", dry_run=dry_run
        )

    hub.log.info(f"Running {cmd} in {cwd} ...")

    proc = subprocess.run(
        cmd,
        shell=shell,
        timeout=timeout,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        universal_newlines=True,
        env=env,
        cwd=cwd,
        **kwargs,
    )
    message = (
        f"Command {cmd} running in {cwd} completed.\n"
        f" Dry-Run: {dry_run}\n"
        f" Exitcode: {proc.returncode}\n"
    )
    if proc.stdout or proc.stderr:
        message += " Process Output:\n"
    if proc.stdout:
        message += f"   >>>>> STDOUT >>>>>\n{proc.stdout}\n   <<<<< STDOUT <<<<<\n"
    if proc.stderr:
        message += f"   >>>>> STDERR >>>>>\n{proc.stderr}\n   <<<<< STDERR <<<<<\n"

    if proc.returncode != 0:
        hub.log.error(message)
    else:
        hub.log.debug(message)

    if fail_on_error:
        proc.check_returncode()

    return data.NamespaceDict(
        retcode=proc.returncode,
        stdout=proc.stdout,
        stderr=proc.stderr,
        dry_run=False,
    )



"""
Create and manage the venvs used for build environments.
"""
import os
from typing import List

__func_alias__ = {"bin_": "bin"}


def bin_(hub, bname: str) -> List[str]:
    """
    Ensure that the desired binary version is present and return the path to
    the python bin to call.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    root = hub.tiamat.cmd.run(["pyenv", "root"], fail_on_error=True).stdout.strip()

    avail = set()
    for line in root.split("\n"):
        avail.add(line.strip())

    python_env = hub.tiamat.virtualenv.init.env(bname)
    if opts.pyenv not in avail:
        if "env" not in python_env:
            python_env.append("env")
        hub.tiamat.cmd.run(
            python_env
            + [
                'PYTHON_CONFIGURE_OPTS="--enable-shared --enable-ipv6"',
                'CONFIGURE_OPTS="--enable-shared --enable-ipv6"',
                "pyenv",
                "install",
                opts.pyenv,
            ],
            fail_on_error=True,
        )
    return python_env + [os.path.join(root, "versions", opts.pyenv, "bin", "python3")]


def create(hub, bname: str):
    """
    Make a virtual environment based on the version of python used to call this script.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    env_bin = hub.tiamat.virtualenv.pyenv.bin(bname)
    cmd = env_bin + [opts.venv_dir, "--clear"]
    if opts.sys_site:
        cmd.append("--system-site-packages")
    hub.tiamat.cmd.run(cmd, fail_on_error=True)



import os
import sys

PKG_BUILDER = "fpm"
if os.name == "nt":
    PKG_BUILDER = "msi"

CLI_CONFIG = {
    "config": {"options": ["-c"], "subcommands": ["_global_"]},
    "name": {"options": ["-n"], "subcommands": ["build"]},
    "directory": {"options": ["-D", "--dir"], "subcommands": ["build", "clean"]},
    "run": {"options": ["-R"], "subcommands": ["build"]},
    "requirements": {"options": ["-r"], "subcommands": ["build"]},
    "exclude": {"options": ["-e"], "subcommands": ["build"], "nargs": "*"},
    "venv_uninstall": {"subcommands": ["build"], "nargs": "*"},
    "system_site": {
        "options": ["-S"],
        "action": "store_true",
        "subcommands": ["build"],
    },
    "pyinstaller_version": {
        "subcommands": ["build"],
    },
    "pyinstaller_runtime_tmpdir": {
        "options": ["--pyinstaller-runtime-tmpdir"],
        "subcommands": ["build"],
    },
    "pyinstaller_args": {
        "subcommands": ["build"],
        "nargs": "*",
    },
    "onedir": {"action": "store_true", "subcommands": ["build"]},
    "pyenv": {"subcommands": ["build"]},
    "no_clean": {"action": "store_true", "subcommands": ["build"]},
    "locale_utf8": {
        "options": ["--locale-utf8"],
        "action": "store_true",
        "subcommands": ["build"],
    },
    "dependencies": {
        "nargs": "*",
        "subcommands": ["build"],
    },
    "omit": {"subcommands": ["build"]},
    "release": {"subcommands": ["build"]},
    "pkg_builder": {"subcommands": ["build"]},
    "pkg_tgt": {"subcommands": ["build"]},
    "srcdir": {"subcommands": ["build"]},
    "system_copy_in": {"nargs": "*", "subcommands": ["build"]},
    "tgt_version": {"subcommands": ["build"]},
    "venv_plugin": {"subcommands": ["build"]},
    "python_bin": {"subcommands": ["build"]},
    "dry_run": {"subcommands": ["build"], "action": "store_true"},
    "timeout": {"subcommands": ["build"]},
    "pip_version": {"subcommands": ["build"]},
}

CONFIG = {
    "config": {
        "default": "",
        "help": (
            "Load extra options from a configuration file, this is useful when the "
            "project needs to use more advanced features like compiling c binaries "
            "into the environment."
        ),
    },
    "dry_run": {
        "default": False,
        "help": "Run the program without actually doing any subprocess calls",
    },
    "name": {
        "default": None,
        "help": "The name of the project to build",
    },
    "directory": {
        "default": os.getcwd(),
        "help": (
            "The path to the directory to build from. This denotes the root of the "
            "python project source tree to work from. This directory should have the "
            "setup.py and the paths referenced in configurations will assume that "
            "this is the root path they are working from."
        ),
    },
    "run": {
        "default": "run.py",
        "help": "The location of the project run.py file",
    },
    "requirements": {
        "default": "requirements.txt",
        "help": "The name of the requirements.txt file to use",
    },
    "exclude": {
        "default": None,
        "help": (
            "A list of exclude files or python modules, these python packages will be "
            "ignored by pyinstaller"
        ),
    },
    "venv_uninstall": {
        "default": [],
        "help": ("List of modules to remove from the virtual environment"),
    },
    "system_site": {
        "default": False,
        "help": (
            "Include the system site-packages when building. This is needed for builds "
            "from custom installs of Python"
        ),
    },
    "pyinstaller_version": {
        "default": "4.0",
        "help": (
            "The version of Pyinstaller to install.  If 'dev' is specified it will be "
            "installed from git, if 'local:/absolute/file/system/path' is specified it "
            "will be installed from the specified file system path"
        ),
    },
    "pyinstaller_runtime_tmpdir": {
        "default": None,
        "help": "Pyinstaller runtime tmpdir",
    },
    "pyinstaller_args": {
        "default": None,
        "help": (
            "Args to pass straight through to pyinstaller. I.E. tiamat build "
            "--pyinstaller-args='--key=1234 -v'"
        ),
    },
    "datas": {"default": None, "help": "Pyinstaller datas mapping."},
    "onedir": {
        "default": False,
        "help": (
            "Instead of producing a single binary produce a directory with all "
            "components"
        ),
    },
    "pyenv": {
        "default": ".".join(str(x) for x in sys.version_info[:3]),
        "help": (
            "Set the python version to build with, if not present the system python "
            "will be used. Only use CPython versions, to see available versions run "
            '`pyenv install --list | grep "3\\.[6789]"`'
        ),
    },
    "no_clean": {
        "default": False,
        "help": (
            "Don't run the clean up sequence, this will leave the venv, spec file and "
            "other artifacts. Only use this for debugging."
        ),
    },
    "locale_utf8": {
        "default": False,
        "help": (
            "Use the UTF-8 locale with PyInstaller, as in PEP538 and PEP540. This "
            "enables UTF-8 on systems which only provide C or POSIX locales."
        ),
    },
    "build": {
        "default": False,
        "help": (
            "Enter in commands to build a non-python binary into the deployed binary. "
            "The build options are set on a named project basis. This allows for "
            "multiple shared binaries to be embedded into the final build."
        ),
    },
    "dependencies": {
        "default": {},
        "help": "comma separated list of dependencies needed for the build",
    },
    "release": {
        "default": {},
        "help": "Release string i.e. '1.el7'. this is the fpm --iteration option.",
    },
    "pkg": {"default": {}, "help": "Options for building packages"},
    "pkg_builder": {
        "default": PKG_BUILDER,
        "help": "Select what package builder plugin to use.",
    },
    "pkg_tgt": {
        "default": None,
        "help": "Specify the os/distribution target to build the package against.",
    },
    "srcdir": {
        "default": None,
        "help": (
            "Instead of reading in a requirements.txt file, install all of the python "
            "package sources and/or wheels found in the specific directory"
        ),
    },
    "system_copy_in": {
        "default": None,
        "help": (
            "A list of directories to copy into the build venv that are not otherwise "
            "detected"
        ),
    },
    "tgt_version": {
        "default": None,
        "help": "Target package version",
    },
    "venv_plugin": {
        "default": "builtin",
        "help": "The python virtual environment plugin to use",
    },
    "python_bin": {
        "default": None,
        "type": str,
        "help": "The path to the python binary to use for system calls",
    },
    "omit": {
        "default": None,
        "help": "File globs to omit so that un-needed files can be removed",
    },
    "timeout": {
        "default": 300,
        "help": ("Amount of time in seconds before considering a command has failed"),
    },
    "pip_version": {
        "default": "latest",
        "help": (
            "The version of pip for tiamat to utilize from pypi.  If 'latest' is "
            "specified it will utilize the latest release from pypi, otherwise specify"
            "the desired version, for example: '20.2.4'"
        ),
    },
}
SUBCOMMANDS = {
    "build": {
        "desc": "Build a dedicated project into an artifact",
        "help": "Build a dedicated project into an artifact",
    },
    "clean": {
        "desc": "Remove the build and dist directories",
        "help": "Remove the build and dist directories",
    },
    "install": {
        "desc": "Download a project from pypi, then build and install it locally",
        "help": "Download a project from pypi, then build and install it locally",
    },
}

# If tiamat is running from a binary, add a subcommand to read it's contents
if getattr(sys, "frozen", False):
    SUBCOMMANDS["freeze"] = {
        "desc": "List the python packages that are bundled in the app",
        "help": "List the python packages that are bundled in the app",
    }

DYNE = {
    "tiamat": ["tiamat"],
}
